var EventEmitter = require('events').EventEmitter;
var exec = require('child_process').exec;
var qs = require('querystring');

function connection(credentials) {

  function run(command, cb) {
    exec(command, function (error, stdout, stderr) {
      try {
        var response = {success: true};
        if (stdout.indexOf('HTTP/1.1 204 NO CONTENT') !== 0) {
          response = JSON.parse(stdout);
        }
        cb(error, response);
      } catch (e) {
        console.log(e);
        console.log(stdout);
        cb(e, null);
      }
    })
    .on('error', function (err) {
      cb(err, null);
    });
  }

  return {
    get: function(url, cb) {
      var command = "curl --user " + credentials + " " + url;
      run(command, cb);
    },

    post: function(url, data, cb) {
      var command = "curl --user " + credentials + " " + url + " --data " + data;
      run(command, cb);
    },

    put: function (url, data, cb) {
      var command = "curl --request PUT --user " + credentials + " " + url + " --data " + data;
      run(command, cb);
    },

    del: function (url, cb) {
      var command = "curl --include --request DELETE --user " + credentials + " " + url;
      run(command, cb);
    }
  };
};

function cacheResult(obj, prop, cb) {
  return function (err, result) {
    if (!err) {
      obj[prop] = result;
    }
    cb(err, result);
  };
}

function Repository(provider, data) {
  this.provider = provider;
  this.scm = data.scm; //  "scm": "git",
  this.hasWiki = data.has_wiki; //": false,
  this.lastUpdated = new Date(data.last_updated); //": "2012-08-21 06:58:33",
  this.creator = data.creator; //": "hgarcia",
  this.forksCount = data.forks_count; //": 0,
  this.createdOn = new Date(data.created_on); //": "2012-08-21 04:58:37",
  this.owner = data.owner; //": "inline",
  this.logo = data.logo; //": null,
  this.emailMailingList = data.email_mailinglist; //": "",
  this.isMq = data.is_mq; //": false,
  this.size = data.size; //": 861586,
  this.readOnly = data.read_only; //": false,
  this.forkOff = data.fork_of; //": null,
  this.mqOf = data.mq_of; //": null,
  this.followersCount = data.followers_count; //": 2,
  this.state = data.state; //": "available",
  this.utcCreatedOn = new Date(data.utc_created_on); //": "2012-08-21 02:58:37+00:00",
  this.website = data.website; //": "",
  this.description = data.description; //": "Web UI for the customers",
  this.hasIssuses = data.has_issues; //": true,
  this.isFork = data.is_fork; //": false,
  this.slug = data.slug; //": "app",
  this.isPrivate = data.is_private; //": true,
  this.name = data.name; //": "app",
  this.language = data.language; //": "javascript",
  this.utcLastUpdated = new Date(data.utc_last_updated); //": "2012-08-21 04:58:33+00:00",
  this.emailWriters = data.email_writers; //": true,
  this.noPublicForks = data.no_public_forks; //": false,
  this.resourceURI = data.resource_uri; //": "/1.0/repositories/inline/app"
  this.getCredentials = function () {
    return this.provider.getCredentials();
  };
  this.getUrl = function () {
    return this.provider.getUrl() + "/repositories/" + this.owner + "/" + this.slug;
  }
}


function BitBucket(options) {
  this.username = options.username;
  this.password = options.password;
  this.root = "api.bitbucket.org";
  this.version = "1.0";
  this.getCredentials = function () {
    return this.username + ":" + this.password;
  };
  this.getUrl = function () {
    return "https://" + this.root + "/" + this.version;
  }
}

BitBucket.prototype.getRepository = function(repository, cb) {
  if (!repository.slug) {return cb(new Error('Repository slug is required.'), null);}
  if (!repository.owner) {return cb(new Error('Repository owner is required.'), null);}
  if (!cb) {return cb(new Error('Callback is required.'), null);}
  var self = this;
  var url = self.getUrl() + "/repositories/" + repository.owner + "/" + repository.slug;
  connection(self.getCredentials()).get(url, function (err, data) {
    cb(err, new Repository(self, data));
  });
};

BitBucket.prototype.repositories = function(cb) {
  var self = this;
  var url = self.getUrl() + "/user/repositories/";
  connection(self.getCredentials()).get(url, cb);
};

Repository.prototype.issue = function (issueId) {
  var self = this;
  return {
    comments: function () {
      var url = self.getUrl() + "/issues/" + issueId + "/comments/";
      return {
        create: function (content, cb) {
          connection(self.getCredentials()).post(url, qs.stringify({content: content}), cb);
        },

        getById: function (id, cb) {
          connection(self.getCredentials()).get(url + id + "/", cb);
        },

        getAll: function (cb) {
          connection(self.getCredentials()).get(url, cb);
        },

        update: function (id, content, cb) {
          connection(self.getCredentials()).put(url + id + "/", qs.stringify({content: content}), cb);
        },

        remove: function (id, cb) {
          connection(self.getCredentials()).del(url + id + "/", cb);
        }
      };
    }
  };
};

Repository.prototype.issues = function () {
  var self = this;
  var url = self.getUrl() + "/issues/";
  return {
    create: function (issue, cb) {
      connection(self.getCredentials()).post(url, qs.stringify(issue), cb);
    },

    getById: function (id, cb) {
      connection(self.getCredentials()).get(url + id + "/", cb);
    },

    get: function (query, cb) {
      var getUrl = url;
      if (!cb) {
        cb = query;
      } else if (query) {
        getUrl = url + '?' + qs.stringify(query);
      }
      connection(self.getCredentials()).get(getUrl, cb);
    },

    update: function (id, issue, cb) {
      connection(self.getCredentials()).put(url + id + "/", qs.stringify(issue), cb);
    },

    remove: function (id, cb) {
      connection(self.getCredentials()).del(url + id + "/", cb);
    }
  };
};

Repository.prototype.milestones = function() {
  var self = this;
  var url = self.getUrl() + "/issues/milestones/";
  return {
    create: function (name, cb) {
      connection(self.getCredentials()).post(url, qs.stringify({name: name}), cb);
    },

    getById: function (id, cb) {
      connection(self.getCredentials()).get(url + id + "/", cb);
    },

    getAll: function (cb) {
      connection(self.getCredentials()).get(url, cb);
    },

    update: function (id, name, cb) {
      connection(self.getCredentials()).put(url + id + "/", qs.stringify({name: name}), cb);
    },

    remove: function (id, cb) {
      connection(self.getCredentials()).del(url + id + "/", cb);
    }
  };
};

Repository.prototype.versions = function() {
  var self = this;
  var url = self.getUrl() + "/issues/versions/";
  return {
    create: function (name, cb) {
      connection(self.getCredentials()).post(url, qs.stringify({name: name}), cb);
    },

    getById: function (id, cb) {
      connection(self.getCredentials()).get(url + id + "/", cb);
    },

    getAll: function (cb) {
      connection(self.getCredentials()).get(url, cb);
    },

    update: function (id, name, cb) {
      connection(self.getCredentials()).put(url + id + "/", qs.stringify({name: name}), cb);
    },

    remove: function (id, cb) {
      connection(self.getCredentials()).del(url + id + "/", cb);
    }
  };
};

Repository.prototype.components = function () {
  var self = this;
  var url = self.getUrl() + "/issues/components/";
  return {
    create: function (name, cb) {
      connection(self.getCredentials()).post(url, qs.stringify({name: name}), cb);
    },

    getById: function (id, cb) {
      connection(self.getCredentials()).get(url + id + "/", cb);
    },

    getAll: function (cb) {
      connection(self.getCredentials()).get(url, cb);
    },

    update: function (id, name, cb) {
      connection(self.getCredentials()).put(url + id + "/", qs.stringify({name: name}), cb);
    },

    remove: function (id, cb) {
      connection(self.getCredentials()).del(url + id + "/", cb);
    }
  };
};

exports.createClient = function (options) {
  if (!options.username) { throw new Error('Username is required'); }
  if (!options.password) { throw new Error('Password is required'); }
  return new BitBucket(options);
};
